from pencil_pusher.documenter import Documenter
import os


def test_gen_local():
    """
    Tests generating a doc with a
    local file
    """
    Documenter.generate("pencil_pusher.documenter", "test")
    assert os.path.isfile("./test.md")
    os.remove("./test.md")



from typing import Any
from pencil_pusher.configs.manager import ConfigManager
from pencil_pusher.configs.base import ConfigParamBase
from pencil_pusher.configs import config_manager


@config_manager.register("title_prefix")
class TitlePrefixConfigParam(ConfigParamBase):
    """
    Processes the title_prefix config parameter

    Note: validation only validates config name
        and config type (string array)
    """

    def on_iterate(self, data: Any):
        """
        This is the validate function
        """
        if data["type"] == ConfigManager.VALIDATE:
            self._validate(data["contents"])
            self.set_defined()

    def _validate(self, contents: dict) -> bool:
        """
        Validates the parameter is included in the
        config file since it is a required field
        """
        if "title_prefix" not in contents:
            # set default value
            contents["title_prefix"] = ""
            return True

        if not isinstance(contents["title_prefix"], str):
            raise ValueError(
                "title_prefix config parameter is not of type str"
            )

        return True



from pencil_pusher.process_util import ProcessUtil


class Documenter:
    @staticmethod
    def install():
        """
        installs the correct pydoc-markdown package
        to the python version being used to execute
        the commands
        """
        ProcessUtil.execute(
            ["python3", "-m", "pip", "install", "pydoc-markdown"]
        )

    @staticmethod
    def generate(module: str, title: str = "", output_dir: str = "./"):
        """
        Generates markdown documentation for a given
        module (.py file) and creates a file with the
        name title as the documentation output

        Args:
            module (str): the name of the module the file is being
                imported from
            title (str): the title of the document file to output
            output_dir (str): the directory to write the output file to
        """
        options = '{renderer: {type: markdown,descriptive_class_title: false,render_toc: true, header_level_by_type: {"Module": 1,"Class": 2,"Method": 3,"Function": 2,"Data": 2}}}'
        if not output_dir.endswith("/"):
            output_dir += "/"

        fp = open(f"{output_dir}{title}.md", "w")
        ProcessUtil.execute(
            ["pydoc-markdown", "-m", f"{module}", options], stdout=fp
        )
        fp.close()
